// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office

package com.ibm.websphere.grid.spi;

/**
 * @ibm-spi
 *
 * <p>
 * SPI is the base class for all Compute Grid system programming interfaces (SPIs).
 * </p>
 */

public abstract class SPI implements Comparable {

	private int _order=0; 	

	/**
	 * Returns the name of this SPI instance. Should return a unique identifier.
	 * @returns logical name of this SPI type.  This method must be 
	 * implemented by the SPI subclass implementor.	
	 */
	public abstract String getName();
	/**
	 * <p>
	 * Called by job scheduler SPI initialization to set invocation order
	 * for this SPI instance.  The order is given by the numeric qualifier
	 * from the xd.spi.properties file for the given SPI type - e.g.
	 * <pre>
	 *    # spi.WSGridNotificationSPI.1=com.abc.NotifySPI_A
	 *    # spi.WSGridNotificationSPI.2=com.abc.NotifySPI_B 
	 * </pre>
	 * Which results in the invocation order being NotifySPI_A, then NotifySPI_B.
	 * Note that each SPI instance of the same logical type should have a 
	 * unique invocation ordinal value. This is not enforced.  SPIs of the same 
	 * type with the same invocation order will be invoked in an unpredictable
	 * order.  
	 * </p>
	 * @param specifies the invocation ordinal value.
	 */
	public void setInvocationOrder(int n) { _order= n; }	
	
	/**
	 * Returns invocation order of this SPI instance.
	 * @return ordinal integer value of invocation order of this SPI instance.
	 */
	public int getInvocationOrder() { return _order; }	
	
	/**
	 * Returns string form of this SPI instance.
	 * @return string form of this SPI instance, in the form:
	 * <pre>
	 * {logical-name}:{invocation-order}
	 * </pre> 
	 */	
	public String toString() {
		return getName()+":"+_order;
	}

	/**
	 * <p>
	 * Compares this SPI instance to another.  Comparisons are numeric,
	 * based on invocation order.
	 * </p>
	 * @param SPI instance to compare with this SPI instance.
	 * @return comparison result as follows:
	 * <ul>
	 * <li>-1 - indicates this SPI invocation order is less than specified SPI invocation order.</li>
	 * <li>0 - indicates this SPI invocation order is equal to the specified SPI invocation order.</li>
	 * <li>1 - indicates this SPI invocation order is greater than specified SPI invocation order.</li>
	 * </ul>
	 */
	public int compareTo(Object o) {
		// TODO Auto-generated method stub
		int otherOrder= ((SPI)o)._order;
		if ( this._order <  otherOrder )
			return -1;
		else if ( this._order == otherOrder )
			return 0;
		else
			return 1;
	}

}
